/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.uinfo.salix;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.dive4elements.artifactdatabase.state.DefaultOutput;
import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifactdatabase.state.FacetActivity;
import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.ArtifactFactory;
import org.dive4elements.artifacts.CallMeta;
import org.dive4elements.artifacts.common.utils.XMLUtils;
import org.dive4elements.river.artifacts.AbstractStaticStateArtifact;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.states.StaticState;
import org.dive4elements.river.artifacts.uinfo.UINFOArtifact;
import org.dive4elements.river.artifacts.uinfo.vegetationzones.VegetationZonesCrossSectionProcessor;
import org.w3c.dom.Document;

/**
 * Display of vegetation zones loaded from database
 *
 * @author Gernot Belger
 *
 */
public class PredefinedVegetationZonesArtifact extends AbstractStaticStateArtifact implements FacetTypes {

    private static final long serialVersionUID = 1L;

    /** Artifact name. */
    private static final String NAME = "predefined_vegetation_zones";

    private static final String STATIC_STATE_NAME = "state.predefined_vegetation_zones.static";

    static {
        // TODO: Move to configuration.
        FacetActivity.Registry.getInstance().register(NAME, FacetActivity.INACTIVE);
    }

    /** The log for this class. */
    private static Logger log = Logger.getLogger(PredefinedVegetationZonesArtifact.class);

    @Override
    public String getName() {
        return NAME;
    }

    @Override
    protected void initStaticState() {

        final StaticState state = new StaticState(STATIC_STATE_NAME);
        final DefaultOutput output = new DefaultOutput("general", "general", "image/png", "chart");

        final List<Facet> facets = getFacets(STATIC_STATE_NAME);
        output.addFacets(facets);
        state.addOutput(output);

        setStaticState(state);
    }

    /**
     * Gets called from factory, to set things up.
     */
    @Override
    public void setup(final String identifier, final ArtifactFactory factory, final Object context, final CallMeta callMeta, final Document data,
            final List<Class> loadFacets) {

        if (log.isDebugEnabled())
            log.debug(XMLUtils.toString(data));

        final String vegZoneData = getDatacageIDValue(data);

        createFacets(callMeta, vegZoneData);

        super.setup(identifier, factory, context, callMeta, data, loadFacets);
    }

    private void createFacets(final CallMeta callMeta, final String vegZoneData) {
        if (vegZoneData == null)
            return;

        addStringData(UINFOArtifact.FIELD_VEGZONES, vegZoneData);

        final List<Facet> facets = new ArrayList<>(1);
        facets.add(VegetationZonesCrossSectionProcessor.createDefaultVegetationZonesCrossSectionFacet(callMeta));
        addFacets(STATIC_STATE_NAME, facets);
    }

    @Override
    protected void initialize(final Artifact artifact, final Object context, final CallMeta meta) {
        // do not clone facets, etc. from master artifact

        log.debug("initialize");
        importData((D4EArtifact) artifact, "river");
        importData((D4EArtifact) artifact, "ld_from");
        importData((D4EArtifact) artifact, "ld_to");

        log.debug("ld_from " + getDataAsString("ld_from"));
        log.debug("ld_to " + getDataAsString("ld_to"));
    }
}